@@ -55,6 +55,16 @@ module IcdGenerator
   $header_files = ["/usr/include/CL/cl.h", "/usr/include/CL/cl_gl.h",
     "/usr/include/CL/cl_ext.h", "/usr/include/CL/cl_gl_ext.h"]
   $windows_header_files = ["/usr/include/CL/cl_dx9_media_sharing.h", "/usr/include/CL/cl_d3d11.h", "/usr/include/CL/cl_d3d10.h"]
+  $cl_data_type_error = { "cl_platform_id"   => "CL_INVALID_PLATFORM",
+                          "cl_device_id"     => "CL_INVALID_DEVICE",
+                          "cl_context"       => "CL_INVALID_CONTEXT",
+                          "cl_command_queue" => "CL_INVALID_COMMAND_QUEUE",
+                          "cl_mem"           => "CL_INVALID_MEM_OBJECT",
+                          "cl_program"       => "CL_INVALID_PROGRAM",
+                          "cl_kernel"        => "CL_INVALID_KERNEL",
+                          "cl_event"         => "CL_INVALID_EVENT",
+                          "cl_sampler"       => "CL_INVALID_SAMPLER"}
+  $non_standard_error = [ "clGetExtensionFunctionAddressForPlatform" ]
   $versions_entries = []
   $buff=20
   $license = <<EOF
@@ -287,7 +297,7 @@ EOF
     File.open('run_dummy_icd_gen.c','w') { |f|
       f.puts generate_run_dummy_icd_source
     }
-    File.open('run_dummy_icd_weak.c','w') { |f|
+    File.open('run_dummy_icd_weak_gen.c','w') { |f|
       f.puts generate_run_dummy_icd_weak_source
     }
   end
@@ -297,6 +307,8 @@ EOF
   # database mode
   def self.generate_ocl_icd_header
     ocl_icd_header = "/**\n#{$license}\n*/\n\n"
+    ocl_icd_header += "#ifndef OCL_ICD_H\n"
+    ocl_icd_header += "#define OCL_ICD_H\n"
     ocl_icd_header += "#pragma GCC diagnostic push\n"
     ocl_icd_header += "#  pragma GCC diagnostic ignored \"-Wcpp\"\n"
     ocl_icd_header += "#  define CL_USE_DEPRECATED_OPENCL_1_0_APIS\n"
@@ -317,7 +329,8 @@ EOF
 	gsub(/\) (CL_API_SUFFIX__VERSION)/m,"\n) \\1").gsub(/\s*$/,'').
 	gsub(/^[\t ]+/,"    ").gsub(/^([^\t ])/, '  \1') + "\n\n"
     }
-    ocl_icd_header += "};\n\n"
+    ocl_icd_header += "};\n"
+    ocl_icd_header += "#endif\n\n"
     return ocl_icd_header
   end
 
@@ -330,8 +343,20 @@ struct func_desc {
   const char* name;
   void(*const addr)(void);
 };
-
+typedef __typeof__(clGetExtensionFunctionAddress) *clGetExtensionFunctionAddress_fn;
 extern const struct func_desc function_description[];
+struct vendor_icd {
+  cl_uint	num_platforms;
+  cl_uint	first_platform;
+  void *	dl_handle;
+  clGetExtensionFunctionAddress_fn ext_fn_ptr;
+};
+struct platform_icd {
+  char *	 extension_suffix;
+  char *	 version;
+  struct vendor_icd *vicd;
+  cl_platform_id pid;
+};
 
 EOF
     ocl_icd_header += "extern struct _cl_icd_dispatch master_dispatch;\n"
@@ -480,7 +505,6 @@ EOF
     ocl_icd_loader_gen_source += "#include \"ocl_icd_loader.h\"\n"
     ocl_icd_loader_gen_source += "#define DEBUG_OCL_ICD_PROVIDE_DUMP_FIELD\n"
     ocl_icd_loader_gen_source += "#include \"ocl_icd_debug.h\"\n"
-    ocl_icd_loader_gen_source += ""
     $api_entries.each { |func_name, entry|
       next if skip_funcs.include?(func_name)
       clean_entry = entry.sub(/(.*\)).*/m,'\1').gsub("/*","").gsub("*/","").gsub("\r","") + "{\n"
@@ -496,12 +520,32 @@ EOF
       end
       fps = first_parameter.split
       ocl_icd_loader_gen_source += "  debug_trace();\n"
-      ocl_icd_loader_gen_source += "  RETURN(((struct _#{fps[0]} *)#{fps[1]})->dispatch->#{func_name}("
+      raise "Unsupported data_type #{fps[0]}" if not $cl_data_type_error[fps[0]]
       ps = parameters.split(",")
       ps = ps.collect { |p|
         p = p.split
         p = p[-1].gsub("*","")
       }
+      error_handler = lambda {
+         if(ps.include?("errcode_ret")) then
+          ocl_icd_loader_gen_source += "    if( errcode_ret != NULL ) {\n";
+          ocl_icd_loader_gen_source += "      *errcode_ret = #{$cl_data_type_error[fps[0]]};\n"
+          ocl_icd_loader_gen_source += "    }\n"
+          ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+        elsif ($non_standard_error.include?(func_name)) then
+          ocl_icd_loader_gen_source += "    RETURN(NULL);\n"
+        else
+          ocl_icd_loader_gen_source += "    RETURN(#{$cl_data_type_error[fps[0]]});\n"
+        end
+      }
+       
+      if(fps[0] == "cl_platform_id") then
+        ocl_icd_loader_gen_source += "  #{fps[1]}=selectPlatformID(#{fps[1]});\n"
+      end
+      ocl_icd_loader_gen_source += "  if( (struct _#{fps[0]} *)#{fps[1]} == NULL) {\n"
+      error_handler.call
+      ocl_icd_loader_gen_source += "  }\n"
+      ocl_icd_loader_gen_source += "  RETURN(((struct _#{fps[0]} *)#{fps[1]})->dispatch->#{func_name}("
       ocl_icd_loader_gen_source += ps.join(", ")
       ocl_icd_loader_gen_source += "));\n"
       ocl_icd_loader_gen_source += "}\n\n"
@@ -552,10 +596,10 @@ EOF
     File.open('ocl_icd.h','w') { |f|
       f.puts generate_ocl_icd_header
     }
-    File.open('ocl_icd_loader.h','w') { |f|
+    File.open('ocl_icd_loader_gen.h','w') { |f|
       f.puts generate_ocl_icd_loader_header
     }
-    File.open('ocl_icd_loader.map','w') { |f|
+    File.open('ocl_icd_loader_gen.map','w') { |f|
       f.puts generate_ocl_icd_loader_map
     }
     File.open('ocl_icd_bindings.c','w') { |f|
