@@ -53,33 +53,17 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 int debug_ocl_icd_mask=0;
 
-typedef __typeof__(clGetExtensionFunctionAddress) *clGetExtensionFunctionAddress_fn;
 typedef __typeof__(clGetPlatformInfo) *clGetPlatformInfo_fn;
 
-
-struct vendor_icd {
-  cl_uint	num_platforms;
-  cl_uint	first_platform;
-  void *	dl_handle;
-  clGetExtensionFunctionAddress_fn ext_fn_ptr;
-};
-
 inline void dump_vendor_icd(const char* info, const struct vendor_icd *v) {
   debug(D_DUMP, "%s %p={ num=%i, first=%i, handle=%p, f=%p}\n", info,
 	v, v->num_platforms, v->first_platform, v->dl_handle, v->ext_fn_ptr);
 }
 
-struct platform_icd {
-  char *	 extension_suffix;
-  char *	 version;
-  struct vendor_icd *vicd;
-  cl_platform_id pid;
-};
-
 struct vendor_icd *_icds=NULL;
 struct platform_icd *_picds=NULL;
 static cl_uint _num_icds = 0;
-static cl_uint _num_picds = 0;
+cl_uint _num_picds = 0;
 
 #ifdef DEBUG_OCL_ICD
 #  define _clS(x) [-x] = #x
@@ -558,7 +542,8 @@ static inline void __attribute__((constructor)) _initClIcd( void ) {
     return;
 #ifdef USE_PTHREAD
   if (in_init) {
-    /* probably reentrency */
+    /* probably reentrency, in_init is a __thread variable */
+    debug(D_WARN, "Executing init while already in init!");
   } else {
     in_init=1;
     __sync_synchronize();
@@ -575,11 +560,13 @@ static inline void __attribute__((constructor)) _initClIcd( void ) {
     in_init=0;
   } else {
     if (in_init) {
-      /* probably reentrency (could also be user threads). */
+      /* probably reentrency (could also be preemptive user-level threads). */
     } else {
-      /* someone else started __initClIcd(). We wait until its end. */
+      /* someone else started __initClIcd(). We wait until it ends. */
       debug(D_WARN, "Waiting end of init");
-      while (!_initialized) ;
+      while (!_initialized) {
+	__sync_synchronize();
+      }
       debug(D_WARN, "Wait done");
    }
   }
@@ -587,6 +574,36 @@ static inline void __attribute__((constructor)) _initClIcd( void ) {
   _initialized = 1;
 }
 
+cl_platform_id __attribute__((visibility("internal")))
+getDefaultPlatformID() {
+  static cl_platform_id defaultPlatformID=NULL;
+  static int defaultSet=0;
+  if (! defaultSet) {
+    do {
+      if(_num_picds == 0) {
+	break;
+      }
+      const char *default_platform = getenv("OPENCL_ICD_DEFAULT_PLATFORM");
+      int num_default_platform;
+      char *end_scan;
+      if (! default_platform) {
+	num_default_platform = 0;
+      } else {
+	num_default_platform = strtol(default_platform, &end_scan, 10);
+	if (*default_platform == '\0' || *end_scan != '\0') {
+	  break;
+	}
+      }
+      if (num_default_platform < 0 || num_default_platform >= _num_picds) {
+	break;
+      }
+      defaultPlatformID=_picds[num_default_platform].pid;
+    } while(0);
+    defaultSet=1;
+  }
+  return defaultPlatformID;
+}
+
 #pragma GCC visibility pop
 #define hidden_alias(name) \
   typeof(name) name##_hid __attribute__ ((alias (#name), visibility("hidden")))
@@ -680,8 +697,11 @@ clGetPlatformIDs(cl_uint          num_entries,
     RETURN(CL_INVALID_VALUE);
   if( num_entries == 0 && platforms != NULL )
     RETURN(CL_INVALID_VALUE);
-  if( _num_icds == 0)
+  if( _num_icds == 0 || _num_picds == 0 ) {
+    if ( num_platforms != NULL )
+      *num_platforms = 0;
     RETURN(CL_PLATFORM_NOT_FOUND_KHR);
+  }
 
   cl_uint i;
   if( num_platforms != NULL ){
@@ -697,6 +717,28 @@ clGetPlatformIDs(cl_uint          num_entries,
 }
 hidden_alias(clGetPlatformIDs);
 
+#define RETURN_WITH_ERRCODE(errvar, errvalue, retvalue) \
+  do { \
+    if(errvar) { \
+      *errvar=errvalue; \
+    } \
+    RETURN(NULL); \
+  } while(0)
+
+#define CHECK_PLATFORM(__pid) \
+  ({ \
+    cl_platform_id _pid=(__pid); \
+    int good=0; \
+    cl_uint j; \
+    for( j=0; j<_num_picds; j++) { \
+      if( _picds[j].pid == _pid) { \
+        good=1; \
+        break; \
+      } \
+    } \
+    good; \
+  })
+
 CL_API_ENTRY cl_context CL_API_CALL
 clCreateContext(const cl_context_properties *  properties ,
                 cl_uint                        num_devices ,
@@ -709,18 +751,29 @@ clCreateContext(const cl_context_properties *  properties ,
   cl_uint i=0;
   if( properties != NULL){
     while( properties[i] != 0 ) {
-      if( properties[i] == CL_CONTEXT_PLATFORM )
+      if( properties[i] == CL_CONTEXT_PLATFORM ) {
+        if((struct _cl_platform_id *) properties[i+1] == NULL) {
+          if(errcode_ret) {
+            *errcode_ret = CL_INVALID_PLATFORM;
+          }
+          RETURN(NULL);
+        } else {
+          if( !CHECK_PLATFORM((cl_platform_id) properties[i+1]) ) {
+	    RETURN_WITH_ERRCODE(errcode_ret, CL_INVALID_PLATFORM, NULL);
+          }
+        }
         RETURN(((struct _cl_platform_id *) properties[i+1])
           ->dispatch->clCreateContext(properties, num_devices, devices,
                         pfn_notify, user_data, errcode_ret));
+      }
       i += 2;
     }
   }
   if(devices == NULL || num_devices == 0) {
-    if(errcode_ret) {
-      *errcode_ret = CL_INVALID_VALUE;
-    }
-    RETURN(NULL);
+    RETURN_WITH_ERRCODE(errcode_ret, CL_INVALID_VALUE, NULL);
+  }
+  if((struct _cl_device_id *)devices[0] == NULL) {
+    RETURN_WITH_ERRCODE(errcode_ret, CL_INVALID_DEVICE, NULL);
   }
   RETURN(((struct _cl_device_id *)devices[0])
     ->dispatch->clCreateContext(properties, num_devices, devices,
@@ -736,25 +789,27 @@ clCreateContextFromType(const cl_context_properties *  properties ,
                         cl_int *                       errcode_ret ){
   debug_trace();
   _initClIcd();
+  if(_num_picds == 0) {
+    goto out;
+  }
   cl_uint i=0;
   if( properties != NULL){
     while( properties[i] != 0 ) {
-      if( properties[i] == CL_CONTEXT_PLATFORM )
-	if (properties[i+1] == 0) {
+      if( properties[i] == CL_CONTEXT_PLATFORM ) {
+	if( (struct _cl_platform_id *) properties[i+1] == NULL ) {
 	  goto out;
+        } else {
+          if( !CHECK_PLATFORM((cl_platform_id) properties[i+1]) ) {
+            goto out;
+          }
         }
         return ((struct _cl_platform_id *) properties[i+1])
           ->dispatch->clCreateContextFromType(properties, device_type,
                         pfn_notify, user_data, errcode_ret);
+      }
       i += 2;
     }
   } else {
-    if(_num_picds == 0) {
-      if(errcode_ret) {
-        *errcode_ret = CL_INVALID_VALUE;
-      }
-      RETURN(NULL);
-    }
     const char *default_platform = getenv("OPENCL_ICD_DEFAULT_PLATFORM");
     int num_default_platform;
     char *end_scan;
@@ -773,10 +828,7 @@ clCreateContextFromType(const cl_context_properties *  properties ,
 	(properties, device_type, pfn_notify, user_data, errcode_ret));
   }
  out:
-  if(errcode_ret) {
-    *errcode_ret = CL_INVALID_PLATFORM;
-  }
-  RETURN(NULL);
+  RETURN_WITH_ERRCODE(errcode_ret, CL_INVALID_PLATFORM, NULL);
 }
 hidden_alias(clCreateContextFromType);
 
@@ -791,10 +843,18 @@ clGetGLContextInfoKHR(const cl_context_properties *  properties ,
   cl_uint i=0;
   if( properties != NULL){
     while( properties[i] != 0 ) {
-      if( properties[i] == CL_CONTEXT_PLATFORM )
+      if( properties[i] == CL_CONTEXT_PLATFORM ) {
+        if( (struct _cl_platform_id *) properties[i+1] == NULL ) {
+	  RETURN(CL_INVALID_PLATFORM);
+        } else {
+          if( !CHECK_PLATFORM((cl_platform_id) properties[i+1]) ) {
+	    RETURN(CL_INVALID_PLATFORM);
+          }
+        }
         RETURN(((struct _cl_platform_id *) properties[i+1])
 	  ->dispatch->clGetGLContextInfoKHR(properties, param_name,
                         param_value_size, param_value, param_value_size_ret));
+      }
       i += 2;
     }
   }
@@ -808,6 +868,8 @@ clWaitForEvents(cl_uint              num_events ,
   debug_trace();
   if( num_events == 0 || event_list == NULL )
     RETURN(CL_INVALID_VALUE);
+  if( (struct _cl_event *)event_list[0] == NULL )
+    RETURN(CL_INVALID_EVENT);
   RETURN(((struct _cl_event *)event_list[0])
     ->dispatch->clWaitForEvents(num_events, event_list));
 }
